//
//	PI CUI Front-end
//

#include "pi.hpp"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int usage()
{
	char app_title[sizeof(__FILE__)];
	char *p = strrchr(strcpy(app_title, __FILE__), '.');
	void(p && memset(p, 0, strlen(p)));
	printf("Usage: %s [Precision] [Filename]\n", app_title);
	return 0;
}

int error(const char *msg)
{
	printf("Error: %s\n", msg);
	return 0;
}

int main(int argc, char *argv[])
{
	if (argc > 3)
		return usage();
	int prec = -1;
	if (argc >= 2) {
		prec = atoi(argv[1]);
		if (prec <= 0)
			puts("Error: Precision must be positive value");
	}
	while (prec <= 0) {
		char buf[80];
		printf("Precision: ");
		fgets(buf, sizeof(buf), stdin);
		sscanf(buf, "%d", &prec);
		if (prec <= 0)
			puts("Error: Precision must be positive value");
	}
	Pi pi;
	if (!pi.create(prec))
		return error("pi.create() failed");
	int n = pi.start();
	if (n == 0)
		return error("pi.start() failed");
	const int LEN = 32;
	int r, c_prev = -1, p_prev = -1;
	while ((r = pi.calc()) != -1) {
		int c = LEN * r / n;
		int p = 100 * r / n;
		if (c != c_prev || p != p_prev) {
			printf("\rCalculating ");
			for (int i = 0; i < LEN; i++)
				putchar(i < c ? '.' : ' ');
			printf(" [%3d%%]", p);
			c_prev = c;
			p_prev = p;
		}
	}
	printf("\rCalculating ");
	for (int i = 0; i < LEN; i++)
		putchar('.');
	puts(" [Done]");
	if (argc == 3) {
		int len = pi.tostr(NULL);
		char *buf = new char[len];
		pi.tostr(buf);
		if (!buf)
			return error("memory allocation failed");
		FILE *fp = fopen(argv[2], "wb");
		if (!fp)
			return error("file creation failed");
		fwrite(buf, 1, strlen(buf), fp);
		fclose(fp);
		free(buf);
	} else {
		int len = pi.tostr(NULL, false);
		char *buf = new char[len];
		pi.tostr(buf, false);
		if (!buf)
			return error("memory allocation failed");
		printf("pi = %s\n", buf);
		free(buf);
	}
	return 0;
}
