//
//	Pi GUI Front-end
//

#include "pi.hpp"

#include <windows.h>
#include <commdlg.h>
#include <commctrl.h>
#include <string.h>

#pragma comment(lib, "comctl32.lib")

#define WM_IDLE		WM_USER

#define WND_CLASS	"PI_GUI"
#define APP_TITLE	"Pi GUI Front-end"

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	enum {
		IDC_STATIC = -1,
		IDC_PREC   = 1001,
		IDC_CALC   = 1002,
		IDC_SAVE   = 1003,
		IDC_PROG   = 1004,
		IDC_RSLT   = 1005
	};
	static HWND hEditPrec, hButtonCalc, hButtonSave;
	static HWND hEditResult, hProgress;
	static HFONT hFontResult;
	static Pi pi;
	static INT nProgMax = 1, nProgCur = 0;
	static bool bCalculating = false;
	static LPSTR lpResult = NULL;
	#define Error(s)	SendMessage(hEditResult, WM_SETTEXT, 0, (LPARAM)"Error: " s)
	switch (uMsg) {
	case WM_IDLE:
		if (bCalculating) {
			int r = pi.calc();
			if (r == -1) {
				SendMessage(hProgress, PBM_SETPOS, (WPARAM)0xFFFF, 0);
				int len = pi.tostr(NULL);
				LPSTR buf = (LPSTR)malloc(len);
				if (!buf)
					Error("Failed to allocate memory");
				else {
					pi.tostr(buf);
					SendMessage(hEditResult, WM_SETTEXT, 0, (LPARAM)buf);
					free(lpResult);
					lpResult = buf;
					EnableWindow(hButtonSave, TRUE);
				}
				bCalculating = false;
				SendMessage(hButtonCalc, WM_SETTEXT, 0, (LPARAM)"Calc");
				EnableWindow(hEditPrec, TRUE);
				return 0;
			}
			if (r * 0xFFFF / nProgMax != nProgCur)
				SendMessage(hProgress, PBM_SETPOS, (WPARAM)(nProgCur = r * 0xFFFF / nProgMax), 0);
		}
		return 0;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_CALC:
			if (bCalculating) {
				bCalculating = false;
				SendMessage(hButtonCalc, WM_SETTEXT, 0, (LPARAM)"Calc");
				EnableWindow(hEditPrec, TRUE);
			} else {
				CHAR buf[20] = {0};
				SendMessage(hEditPrec, WM_GETTEXT, (WPARAM)sizeof(buf), (LPARAM)buf);
				int prec = atoi(buf);
				if (prec < 0) {
					Error("Precision must be positive value");
					break;
				}
				if (!pi.create(prec)) {
					Error("Failed to prepare calculation");
					break;
				}
				nProgMax = pi.start();
				if (nProgMax == 0) {
					Error("Failed to start calculation");
					break;
				}
				SendMessage(hProgress, PBM_SETPOS, (WPARAM)(nProgCur = 0), 0);
				SendMessage(hButtonCalc, WM_SETTEXT, 0, (LPARAM)"Abort");
				EnableWindow(hEditPrec, FALSE);
				bCalculating = true;
			}
			break;
		case IDC_SAVE:
			if (lpResult) {
				CHAR szFile[MAX_PATH] = {0};
				OPENFILENAME ofn = { sizeof(OPENFILENAME), };
				ofn.hwndOwner    = hWnd;
				ofn.lpstrFilter  = "All File (*.*)\0" "*.*\0";
				ofn.nFilterIndex = 1;
				ofn.lpstrFile    = strcpy(szFile, "pi.txt");
				ofn.nMaxFile     = sizeof(szFile);
				ofn.Flags        = OFN_OVERWRITEPROMPT;
				if (!GetSaveFileName(&ofn))
					break;
				HANDLE hFile = CreateFile(szFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
				if (hFile == INVALID_HANDLE_VALUE)
					break;
				DWORD dwLen = (DWORD)strlen(lpResult);
				WriteFile(hFile, lpResult, dwLen, &dwLen, NULL);
				CloseHandle(hFile);
			}
			break;
		}
		break;
	case WM_CREATE:
		{
			HINSTANCE hInstance = (HINSTANCE)lParam;
			RECT rc;
			GetClientRect(hWnd, &rc);
			LONG cx = rc.right - rc.left, cy = rc.bottom - rc.top;
			HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
			LOGFONT lf = {0};
			GetObject(hFont, sizeof(LOGFONT), &lf);
			lf.lfPitchAndFamily = FIXED_PITCH;
			lf.lfHeight = 13;
			hFontResult = CreateFontIndirect(&lf);
			HWND hLabel = CreateWindowEx(0, "STATIC", "Precision:", WS_CHILD | WS_VISIBLE,
							20, 15, 60, 15, hWnd, (HMENU)IDC_STATIC, hInstance, NULL);
			SendMessage(hLabel, WM_SETFONT, (WPARAM)hFont, TRUE);
			hEditPrec   = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", "1000", WS_CHILD | WS_VISIBLE | ES_RIGHT,
							80, 10, 60, 20, hWnd, (HMENU)IDC_PREC, hInstance, NULL);
			SendMessage(hEditPrec, WM_SETFONT, (WPARAM)hFont, TRUE);
			hButtonCalc = CreateWindowEx(0, "BUTTON", "Calc", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
							150, 10, 80, 20, hWnd, (HMENU)IDC_CALC, hInstance, NULL);
			SendMessage(hButtonCalc, WM_SETFONT, (WPARAM)hFont, TRUE);
			hButtonSave = CreateWindowEx(0, "BUTTON", "Save...", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
							240, 10, 60, 20, hWnd, (HMENU)IDC_SAVE, hInstance, NULL);
			SendMessage(hButtonSave, WM_SETFONT, (WPARAM)hFont, TRUE);
			hEditResult = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", NULL, WS_CHILD | WS_VISIBLE |
							ES_MULTILINE | ES_AUTOHSCROLL | ES_AUTOVSCROLL | WS_HSCROLL | WS_VSCROLL,
							10, 40, cx - 20, cy - 75, hWnd, (HMENU)IDC_RSLT, hInstance, NULL);
			SendMessage(hEditResult, WM_SETFONT, (WPARAM)hFontResult, TRUE);
			hProgress   = CreateWindowEx(0, PROGRESS_CLASS, NULL, WS_CHILD | WS_VISIBLE,
							10, cy - 25, cx - 20,  15, hWnd, (HMENU)IDC_PROG, hInstance, NULL);
			SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, 0xFFFF));
			EnableWindow(hButtonSave, FALSE);
		}
		break;
	case WM_DESTROY:
		{
			free(lpResult);
			DeleteObject(hFontResult);
		}
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int nCmdShow)
{
	InitCommonControls();
	WNDCLASSEX wcx    = { sizeof(WNDCLASSEX), };
	wcx.hInstance     = hInstance;
	wcx.lpszClassName = WND_CLASS;
	wcx.lpfnWndProc   = WndProc;
	wcx.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
	wcx.hCursor       = LoadCursor(NULL, IDC_ARROW);
	RegisterClassEx(&wcx);
	HWND hWnd = CreateWindowEx(0, wcx.lpszClassName, APP_TITLE, WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
	                           CW_USEDEFAULT, CW_USEDEFAULT, 800, 600, NULL, NULL, wcx.hInstance, hInstance);
	ShowWindow(hWnd, nCmdShow);
	MSG msg;
	do {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		WndProc(hWnd, WM_IDLE, 0, 0);
	} while (msg.message != WM_QUIT);
	return (int)msg.wParam;
}
